//
// ETSI (European Telecommunications Standards Institute, referred to as ETSI)
// ARF - ETSI ISG Augmented Reality Framework (ISG ARF)
// (C) 2022
//
// Development "World Storage", data management, authoring tools
//
// Authors:
// - Sylvain Renault (Fraunhofer HHI)
// 
// Date: Feb. 2022
//

#define USING_OPENAPI_GENERATOR // alt. is Swagger

using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEditor;
using ETSI.ARF.WorldStorage.RESTful;

#if USING_OPENAPI_GENERATOR
using Org.OpenAPITools.Api;
using Org.OpenAPITools.Model;
#else
using IO.Swagger.Api;
using IO.Swagger.Model;
#endif


namespace ETSI.ARF.WorldStorage.UI
{
    public class TrackableWindow : EditorWindow
    {
        [HideInInspector] public WorldStorageSettings worldStorageSettings;

        [SerializeField] public List<string> trackables = new List<string>();

        bool groupEnabled;

        string UUID = System.Guid.Empty.ToString();
        string customName = "NotDefined";
        string creatorUUID = System.Guid.Empty.ToString();
        string type = "Unknow";
        string unit = "Unknow";
        Vector2Int dim;

        private Trackable currentTrackable;
        private Vector2 scrollPos;
        private Color ori;
        private GUIStyle gsTest;

        public static void ShowWindow(WorldStorageSettings ws)
        {
            TrackableWindow win = EditorWindow.GetWindow(typeof(TrackableWindow), false, WorldStorageWindow.winName) as TrackableWindow;
            win.worldStorageSettings = ws;
        }

        public TrackableWindow()
        {
            // init somne stuffs
            //currentTrackable = new Trackable();
        }

        void OnGUI()
        {
            ori = GUI.backgroundColor; // remember ori color

            gsTest = new GUIStyle("window");
            gsTest.normal.textColor = WorldStorageWindow.arfColors[0];
            gsTest.fontStyle = FontStyle.Bold;

            scrollPos = EditorGUILayout.BeginScrollView(scrollPos, GUILayout.ExpandWidth(true));
            WorldStorageWindow.DrawCopyright();

            DrawTrackableStuffs();

            EditorGUILayout.EndScrollView();

            if (GUILayout.Button("Close Window"))
            {
                Close();
            }
        }

        void DrawTrackableStuffs()// Trackable trackable)
        {
            GUILayout.BeginVertical("AR Trackable", gsTest);
            //
            GUILayout.Label("Server: " + worldStorageSettings.serverName, EditorStyles.whiteLargeLabel);
            EditorGUILayout.Space();

            //GUILayout.BeginHorizontal();
            customName = EditorGUILayout.TextField("Name of Trackable", customName);
            GUILayout.Label("UUID: " + UUID, EditorStyles.miniLabel); // readonly
            GUILayout.Label("Creator UID: " + creatorUUID, EditorStyles.miniLabel); // readonly
            EditorGUILayout.Space();

            GUI.backgroundColor = WorldStorageWindow.arfColors[1];
            if (GUILayout.Button("Get Parameters"))
            {
                if (!customName.Contains("[")) UUID = customName;
                else
                {
                    // extract the UUID
                    UUID = customName.Split('[', ']')[1];
                }
                Trackable obj = RESTfulTrackableRequest.GetTrackable(worldStorageSettings, UUID);
                customName = obj.Name;
                creatorUUID = obj.CreatorUUID.ToString();
                type = obj.TrackableType.ToString();
                unit = obj.Unit.ToString();
            }
            GUI.backgroundColor = ori;

            type = EditorGUILayout.TextField("Trackable Type", type);
            unit = EditorGUILayout.TextField("Unit System", unit);

            EditorGUILayout.Space(10);
            dim = EditorGUILayout.Vector2IntField("Dimension", dim);

            EditorGUILayout.Space();
            GUILayout.Button("Payload from Asset...");

            EditorGUILayout.Space();
            groupEnabled = EditorGUILayout.BeginToggleGroup("Optional Parameters:", groupEnabled);
            EditorGUILayout.IntField("Number of KeyValues", 0);
            EditorGUILayout.Space();
            EditorGUILayout.TextField("Key", "");
            EditorGUILayout.TextField("Value", "");
            EditorGUILayout.EndToggleGroup();
            //
            GUILayout.EndVertical();

            // ###########################################################
            GUI.backgroundColor = WorldStorageWindow.arfColors[2];
            if (GUILayout.Button("Create/Update Trackable"))
            {
                Debug.Log("PostAddTrackable");
                
                if (string.IsNullOrEmpty(UUID) || UUID == "0") UUID = System.Guid.Empty.ToString();
                Trackable obj = RESTfulTrackableRequest.TrackableFromStrings(UUID, customName, worldStorageSettings.creatorUID);
                RESTfulTrackableRequest.AddTrackable(worldStorageSettings, obj);
                WorldStorageWindow.WorldStorageWindowSingleton.UpdateListTrackables();
                WorldStorageWindow.WorldStorageWindowSingleton.Repaint();

                UUID = obj.UUID.ToString();
                customName = obj.Name;
                type = obj.TrackableType.ToString();
                unit = obj.Unit.ToString();
            }

            // ###########################################################
            GUI.backgroundColor = WorldStorageWindow.arfColors[3];
            if (GUILayout.Button("Delete Trackable"))
            {
                Debug.Log("Deleting Trackable");
                RESTfulTrackableRequest.DeleteTrackable(worldStorageSettings, UUID);
                UUID = System.Guid.Empty.ToString();
                creatorUUID = System.Guid.Empty.ToString();
                type = "";
                unit = "";
                WorldStorageWindow.WorldStorageWindowSingleton.UpdateListTrackables();
                WorldStorageWindow.WorldStorageWindowSingleton.Repaint();
            }
            GUI.backgroundColor = ori;

            // ###########################################################
            GUI.backgroundColor = WorldStorageWindow.arfColors[5];
            if (GUILayout.Button("Generate GameObject Component"))
            {
            }
            GUI.backgroundColor = ori;
        }
    }
}